package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ITaskTimeService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.TaskTime;
import cn.estsh.i3plus.pojo.platform.repository.TaskTimeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 定时任务业务接口实现
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:43.974
 * @Modify      :
 **/
@Service
public class TaskTimeServiceImpl implements ITaskTimeService {

    public static final Logger LOGGER = LoggerFactory.getLogger(TaskTimeServiceImpl.class);

    @Autowired
    private TaskTimeRepository taskTimeRDao;

    @Override
    public void updateTaskTime(TaskTime taskTime) {
        LOGGER.info("定时任务 TASK_TIME :{}", taskTime);
        taskTimeRDao.update(taskTime);
    }

    @Override
    public void updateTaskTimeStatus(String id, int status, SessionUser user) {
        LOGGER.info("定时任务 SYS_MENU  id:{} status:{} user:{}", id,status,user);
        TaskTime taskTime = taskTimeRDao.getById(Long.parseLong(id));
        taskTime.setTaskStatus(status);
        taskTime.setModifyUser(user.getUserName());
        taskTimeRDao.update(taskTime);
    }

    @Override
    public void deleteTaskTimeById(String id) {
        LOGGER.info("定时任务 TASK_TIME Key:{}", id);
        taskTimeRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public TaskTime insertTaskTime(TaskTime taskTime) {
        LOGGER.info("定时任务 TASK_TIME :{}", taskTime);
        return taskTimeRDao.insert(taskTime);
    }

    @Override
    public List<TaskTime> listTaskTime() {
        LOGGER.info("定时任务 TASK_TIME find All");
        return taskTimeRDao.findAll();
    }

    @Override
    public TaskTime getTaskTimeById(String id) {
        LOGGER.info("定时任务 TASK_TIME find id:{}", id);
        return taskTimeRDao.getOne(Long.parseLong(id));
    }

    @Override
    public ListPager queryTaskTimeByPager(TaskTime taskTime, Pager pager) {
        LOGGER.info("定时任务 TASK_TIME TaskTime:{}，Pager:{}",taskTime,pager);
        if(taskTime == null) {
            pager = PagerHelper.getPager(pager, taskTimeRDao.listCount());
            return new ListPager(taskTimeRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlTaskTime(taskTime);
            pager = PagerHelper.getPager(pager, taskTimeRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(taskTimeRDao.findByHqlWherePage(hqlPack + taskTime.orderBy(),pager),pager);
        }
    }

    @Override
    public void deleteTaskTimeByIds(String[] ids) {
        LOGGER.info("定时任务 TASK_TIME ids:{}", ids.toString());
        // 转换类型
        Long[] array = new Long[ids.length];
        for (int i = 0; i < ids.length; i++) {
            array[i] = Long.parseLong(ids[i]);
        }

        taskTimeRDao.deleteByIds(array);
    }

    @Override
    public void updateTaskTimeStatusByIds(String[] ids, Integer status) {
        LOGGER.info("定时任务 TASK_TIME ids：{}，status：{}",ids,status);
        StringBuffer where = new StringBuffer();
        HqlPack.getInPack(String.join(",", ids), "id", where);

        taskTimeRDao.updateByHqlWhere(where.toString(), "taskStatus", status);
    }


}
